@@ -1,5 +1,5 @@
 package DBIx::Pg::CallFunction;
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 
 =head1 NAME
 
@@ -7,7 +7,7 @@ DBIx::Pg::CallFunction - Simple interface for calling PostgreSQL functions from
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
@@ -48,13 +48,13 @@ but with different input argument types.
 =head1 SEE ALSO
 
 This module is built on top of L<DBI>, and
-you need to use that module (and the appropriate DBD::xx drivers)
+you need to use that module (and the appropriate DBD::Pg driver)
 to establish a database connection.
 
 There is another module providing about the same functionality,
-but without support for named arguments. Have a look at this one
-if you need to access functions without named arguments,
-or if you are using Oracle:
+but without support for named arguments for PostgreSQL.
+Have a look at this one if you need to access functions
+without named arguments, or if you are using Oracle:
 
 L<DBIx::ProcedureCall|DBIx::ProcedureCall>
 
@@ -133,66 +133,84 @@ sub _proretset
     # If 0, the function always returns exactly one row.
     my ($self, $name, $argnames) = @_;
 
-    my $get_proretset = $self->{dbh}->prepare_cached("
-        WITH
-        -- Unnest the proargname and proargmode
-        -- arrays, so we get one argument per row,
-        -- allowing us to select only the IN
-        -- arguments and build new arrays.
-        NamedInputArgumentFunctions AS (
-            -- For functions with only IN arguments,
-            -- proargmodes IS NULL
-            SELECT
-                oid,
-                proname,
-                proretset,
-                unnest(proargnames) AS proargname,
-                'i'::text AS proargmode
+    my $get_proretset;
+    if (@$argnames == 0)
+    {
+        # no arguments
+        $get_proretset = $self->{dbh}->prepare_cached("
+            SELECT proretset
             FROM pg_catalog.pg_proc
-            WHERE proargnames IS NOT NULL
+            WHERE proname = ?::text
+            AND proargnames IS NULL
             AND proargmodes IS NULL
-            UNION ALL
-            -- For functions with INOUT/OUT arguments,
-            -- proargmodes is an array where each
-            -- position matches proargname and
-            -- indicates if its an IN, OUT or INOUT
-            -- argument.
-            SELECT
-                oid,
-                proname,
-                proretset,
-                unnest(proargnames) AS proargname,
-                unnest(proargmodes) AS proargmode
-            FROM pg_catalog.pg_proc
-            WHERE proargnames IS NOT NULL
-            AND proargmodes IS NOT NULL
-        ),
-        OnlyINandINOUTArguments AS (
-            -- Select only the IN and INOUT
-            -- arguments and build new arrays
-            SELECT
-                oid,
-                proname,
-                proretset,
-                array_agg(proargname) AS proargnames
-            FROM NamedInputArgumentFunctions
-            WHERE proargmode IN ('i','b')
-            GROUP BY
-                oid,
-                proname,
-                proretset
-        )
-        -- Find any function matching the name
-        -- and having identical argument names
-        SELECT * FROM OnlyINandINOUTArguments
-        WHERE proname = ?::text
-        -- The order of arguments doesn't matter,
-        -- so compare the arrays by checking
-        -- if A contains B and B contains A
-        AND ?::text[] <@ proargnames
-        AND ?::text[] @> proargnames
-    ");
-    $get_proretset->execute($name, $argnames, $argnames);
+            AND pronargs = 0
+        ");
+        $get_proretset->execute($name);
+    }
+    else
+    {
+        $get_proretset = $self->{dbh}->prepare_cached("
+            WITH
+            -- Unnest the proargname and proargmode
+            -- arrays, so we get one argument per row,
+            -- allowing us to select only the IN
+            -- arguments and build new arrays.
+            NamedInputArgumentFunctions AS (
+                -- For functions with only IN arguments,
+                -- proargmodes IS NULL
+                SELECT
+                    oid,
+                    proname,
+                    proretset,
+                    unnest(proargnames) AS proargname,
+                    'i'::text AS proargmode
+                FROM pg_catalog.pg_proc
+                WHERE proargnames IS NOT NULL
+                AND proargmodes IS NULL
+                UNION ALL
+                -- For functions with INOUT/OUT arguments,
+                -- proargmodes is an array where each
+                -- position matches proargname and
+                -- indicates if its an IN, OUT or INOUT
+                -- argument.
+                SELECT
+                    oid,
+                    proname,
+                    proretset,
+                    unnest(proargnames) AS proargname,
+                    unnest(proargmodes) AS proargmode
+                FROM pg_catalog.pg_proc
+                WHERE proargnames IS NOT NULL
+                AND proargmodes IS NOT NULL
+            ),
+            OnlyINandINOUTArguments AS (
+                -- Select only the IN and INOUT
+                -- arguments and build new arrays
+                SELECT
+                    oid,
+                    proname,
+                    proretset,
+                    array_agg(proargname) AS proargnames
+                FROM NamedInputArgumentFunctions
+                WHERE proargmode IN ('i','b')
+                GROUP BY
+                    oid,
+                    proname,
+                    proretset
+            )
+            -- Find any function matching the name
+            -- and having identical argument names
+            SELECT * FROM OnlyINandINOUTArguments
+            WHERE proname = ?::text
+            -- The order of arguments doesn't matter,
+            -- so compare the arrays by checking
+            -- if A contains B and B contains A
+            AND ?::text[] <@ proargnames
+            AND ?::text[] @> proargnames
+        ");
+        $get_proretset->execute($name, $argnames, $argnames);
+    }
+
 
     my $proretset;
     my $i = 0;
@@ -220,6 +238,11 @@ sub call
 
     my $validate_name_regex = qr/^[a-zA-Z_][a-zA-Z0-9_]*$/;
 
+    unless (defined $args)
+    {
+        $args = {};
+    }
+
     croak "dbh and name must be defined" unless defined $self->{dbh} && defined $name;
     croak "invalid format of name" unless $name =~ $validate_name_regex;
     croak "args must be a hashref" unless ref $args eq 'HASH';
